package theater

import (
	"context"
	"time"

	"github.com/sirupsen/logrus"

	"gitlab.com/oiacow/goheroes/backend/core"
	"gitlab.com/oiacow/goheroes/backend/matchmaking"
	"gitlab.com/oiacow/goheroes/backend/storage/database"
	"gitlab.com/oiacow/goheroes/backend/storage/kvstore"
)

var (
	// thtrGLST = "GLST"
	thtrKICK = "KICK"
	thtrPLVT = "PLVT"
	thtrUBRA = "UBRA"
)

// Theater Handles incoming and outgoing theater communication
type Theater struct {
	db        database.Adapter
	mm        *matchmaking.Pool
	socket    *core.Socket
	socketUDP *core.SocketUDP
}

// New creates and starts a new TheaterManager
func New(bind string, db database.Adapter, kvs *kvstore.Storage, mm *matchmaking.Pool) *Theater {
	socket, err := core.NewSocketTCP(bind)
	if err != nil {
		logrus.Fatal(err)
		return nil
	}

	socketUDP, err := core.NewSocketUDP(bind)
	if err != nil {
		logrus.Fatal(err)
		return nil
	}

	tm := &Theater{
		mm:        mm,
		db:        db,
		socket:    socket,
		socketUDP: socketUDP,
	}

	return tm
}

func (tm *Theater) ListenAndServe(ctx context.Context) {
	go tm.Run(ctx)
}

func (tm *Theater) Run(ctx context.Context) {
	for {
		select {
		case event := <-tm.socketUDP.EventChan:
			tm.handleUDP(event)
		case event := <-tm.socket.EventChan:
			tm.handleTLS(event)
		case <-ctx.Done():
			return
		}
	}
}

func (tm *Theater) handleUDP(event core.SocketUDPEvent) {
	switch event.Name {
	case "ECHO":
		tm.ECHO(event)
	default:
		logrus.
			WithFields(logrus.Fields{
				"event": event.Name,
				"data":  event.Data,
			}).
			Warn("theater.UnhandledRequest (UDP)")
	}
}

func (tm *Theater) handleTLS(event core.SocketEvent) {
	ev, ok := event.Data.(core.EClientCmd)
	if !ok {
		logrus.Error("Logic error: Cant cast event to core.EClientCmd")
		return
	}

	// if !ev.Client.IsActive {
	// 	logrus.WithField("cmd", ev.Cmd).Warn("Inactive client")
	// 	return
	// }

	switch event.Name {
	case "newClient":
		tm.newClient(ev.Client)
	case "client.cmd.CONN":
		tm.Connect(ev)
	case "client.cmd.USER":
		tm.Login(ev)
	case "client.cmd.LLST":
		tm.GetLobbyList(ev)
		tm.LobbyData(ev)
	case "client.cmd.GDAT":
		tm.GameData(ev)
	case "client.cmd.EGAM":
		tm.EnterGame(ev)
	case "client.cmd.ECNL":
		tm.EnterConnLost(ev)
	case "client.cmd.CGAM":
		tm.CreateGame(ev)
	case "client.cmd.UBRA":
		tm.UpdateBracket(ev)
	case "client.cmd.UGAM":
		tm.UpdateGameData(ev)
	case "client.cmd.EGRS":
		tm.EnterGameHostResponse(ev)
	case "client.cmd.PENT":
		tm.PlayerEntered(ev)
	case "client.cmd.PLVT":
		tm.PlayerExited(ev)
	case "client.cmd.UPLA":
		tm.UpdatePlayerData(ev)
	case "client.cmd.PING":
		// TODO: Use metrics in the response and save it for later use.
		return
	default:
		logrus.
			WithFields(logrus.Fields{
				"event":   event.Name,
				"query":   ev.Cmd.Query,
				"payload": ev.Cmd.Msg,
			}).
			Warn("theater.UnhandledRequest")
	}
}

func (tm *Theater) newClient(client *core.Client) {
	// Start Heartbeat
	client.Ticker = time.NewTicker(time.Second * 15)
	go func() {
		for {
			if !client.IsActive {
				return
			}
			select {
			case <-client.Ticker.C:
				if !client.IsActive {
					return
				}
				tm.PING(client)
			}
		}
	}()
}
